from util import *
import pr
import patterns
from specs import SPECIFIERS
from te_gen_util import get_type
import os.path
import config
from domain_model import setup_normalized_transitions
import presentations
import walk
from textentry import group_name_map

####################### Record Actions ###########################

history = []

def post (obj, relation, value):
    history.append([obj, relation, value])
    return 'done'


##################### Callbacks #####################################

def adjust_pattern_list_finalCB ():
    """we look at active status of pattern realizations
    and adjust the type contributors"""
    for p in pr.all_pattern_realizations.values():
        patname = get_type(p)[:-1]
        contributor = patname.lower()
        patobj = eval('patterns.' + patname)
        participants = patobj.participants
        can_modify_type = patobj.gen_resource_types
        if can_modify_type != 'no':
            for part in participants:
                obj = p.__dict__[part] # the resource object
                if obj.__class__.__name__ == 'File':
                    if p.active == 0 and contributor in obj.type_contributors:
                        obj.type_contributors.remove(contributor)
                        obj.security_context = obj.original_context
                    elif p.active == 1 and contributor not in obj.type_contributors:
                        obj.type_contributors.append(contributor)




def set_integrity_groupCB(process_name, group):
    #print "setting " + process_name + " to group " + group
    resources_dict[process_name].group = group

def set_active_valueCB (item, value):
    if value == 'OFF':
        # delete pattern instance
        adjust_pattern_active_aux(item, 0)
    else:
        adjust_pattern_active_aux(item, 1)

def adjust_pattern_active_aux(uid, val):
    """this changes the active status of pattern realizations"""
    ## need to align settings with patterns
    ##uids are of form pa<nbr>
    ##pattern_assertion keys are the print_forms
    bad = pr.pattern_assertions[uid2name[uid]]
    bad.accepted = val
    for e in bad.evidence:
        #print pr.all_pattern_realizations[e]
        there = pr.all_pattern_realizations[e].active
        if there != val:
            post(pr.all_pattern_realizations[e], 'active', val)
        pr.all_pattern_realizations[e].active = val

def set_file_contextsCB (group, val):
       if val != 'ignore':
        processes = [p for p in resources_dict.values() if p.__class__.__name__ == 'Process' and \
                     p.group == group]
        files = []
        if val in ['ALL files read', 'ALL files read or modified']:
            for p in processes:
                #print p
                there = [t.domain for t in p.in_edges  \
                         if t.domain.__class__.__name__ == 'File']
                files = files + there
        elif val in ['ALL files modified', 'ALL files read or modified']:
            for p in processes:
                there = [t.range for t in p.out_edges \
                         if t.range.__class__.__name__ == 'File']
                files = files + there
        for f in seq2set(files):
            post(f, 'type_contributors', group)
            f.type_contributors.append(group)

def set_scCB (name, value, return_to_original = 0):
    f = resources_dict[name]
    TS.set_sc_for_objCB(f, value)

def set_dir_scCB (name, value):
    obj = resources_dict[name]
    post(obj, 'sc_extent', value)
    obj.sc_extent = value

######################### Set types ###########################################
    


class TypingStrategy:
    def set_sc_for_objCB(self, f, value):
            keys = group_name_map.keys()
            sc  = user_and_role(f.original_context) + PRGM.name.lower() + '_'
            mutable = 0
            if f.sc_from == 'typegen':
                mutable = 1
            # suppose not mutable and user chooses to override original. We will need to double check these 
            if value == 'all':
                f.type_contributors.sort()
                for item in f.type_contributors: # patterns and groups
                    if item in keys: # is item a group name
                        item = group_name_map[item]
                        #else item is the name of a pattern - must be a pattern that forces resource type changes -
                        # otherwise we would not be here
                    sc = sc + item + '_'
                sc = sc + 't'
            elif value in f.type_contributors:
                if value in keys:
                    value = group_name_map[value]
                sc = sc + value + "_t"
            else: # user wants original to stand
                f.type_contributors = []
                sc = f.original_context
            post(f, 'security_context', sc)
            f.security_context = sc

TS = TypingStrategy()





##################### Generators #########################
    



### typing rules
## original context could be set by environment or equal the polgen default
## if the polgen default then it is possible that the type could be changed thru participation in a pattern
##      the new type would be <prgm-name>_<pattern-type>_t
## or thru belonging to a group
##      the new type would be <prgm-name>-<group-name>_t
## environment types are not reset by polgen -
##    that is, we only want to change types for those files that are "owned" by the application
##    if we changed a type from 1_t to 2_t, then every other application that used to communicate
##    with 1_t would now also need to communicate with 2_t - should we allow the policy writer to do this?
### let's say we allow the policy writer to change what he/she wants based on information that polgen provides.
##     two situations
##  1. patterns - writer is notified that if an environmental resource is treated as special there would be a pattern
##  2. groups of read or modified - default is to bring up the original environment set type.  If policy writer chooses
##    to override the default, we pop up a message - " are you sure you want to do this"
##


        
##################### Fc Generation ###############################

def get_dir_extension_to_use (name, extension):
    dir = os.path.dirname(name)
    file_name = name.split('/')[-1].split('.')
    possible_name = ""
    if extension == 'dir/name':
        use = name
    elif len(file_name) > 1:
        possible_name = file_name[0]
        if extension == 'dir/name.*':
            use = dir + '/' + possible_name + '.*'
        else:
            use = dir + extension[3:]
    else: # no . in string
        if extension == 'dir/name.*':
            use = name + '.*'
        else:
            use = dir + extension[3:]
    return use

def do_fc_generation(name_extension_pairs):
    prgm = get_current_program()
    pname = prgm.name
    fileid  = pname + ".fc"
    fileStr = config.result_file(fileid)
    out = open(fileStr, "w")
    fc_strm = open(fileStr, "w")
    print "Writing to " + fileStr
    responses = []
    for item in name_extension_pairs:
        name = item[0]
        obj = resources_dict[name]
        context = obj.security_context
        extension = item[1]
        use = get_dir_extension_to_use(name, extension)
        responses.append([use, context])
    responses = seq2set(responses)
    max = 0
    for  r in responses:
        if len(r[0]) > max:
            max = len(r[0])
    for r in responses:
        fname = r[0]
        context = r[1]
        there = len(fname)
        spaces = max - there + 40
        fc_strm.write("%s %*s \n" % (fname, spaces, context))
    fc_strm.close()

################## Te Generation Strategies ########################
    
class GenerationStrategy:

       
    def do_te_generation (self):
        self.open_file()
        self.do_type_declarations()
        self.do_allows_and_macros(self.file, 0)
        self.close_file()
        
    def open_file (self):
        prgm = get_current_program()
        pname = prgm.name
        fileid  = pname + ".te"
        fileStr = config.result_file(fileid)
        out = open(fileStr, "w")
        print "Writing to " + fileStr
        self.file = out
        
    def do_type_declarations (self):
        spec_out = self.file
        spec_out.write("##################### Type Declarations #################### \n\n")
        use = []
        found = {}
        for r in resources_dict.values():
            #print r
            orig = r.original_context
            now = r.security_context
            if orig != now:
                if now in found.keys():
                    found[now].append(r)
                else:
                    found[now] = [r]
        for sc in found.keys():
            types = {}
            for support in found[sc]:
                ty = support.__class__.__name__.lower()
                if support.special_name != None:
                    ty = support.special_name
                if ty in types.keys():
                    types[ty].append(support)
                else:
                    types[ty] = [support]
            for t in types.keys():
                if t == 'spawn':
                    continue
                elif t in ['file', 'dir']:
                    if sc.find('exec_t') != -1:
                        attribute = 'file_type, exec_type'
                    else:
                        attribute = 'file_type'
                elif t == 'process':
                    attribute = 'domain'
                else:
                    attribute = t
    
                use.append('type ' + str(sc).split(':')[-1] + ',' \
                                      + attribute + '; \n\n')
        use = seq2set(use)
        for item in use:
            spec_out.write(item)

    
    def close_file(self):
        print 'closing file'
        self.file.close()
        
    def do_allows_and_macros (self, strm, test_for_coveragep):
        return "done"
    
class NoMacros (GenerationStrategy):
    
    def do_allows_and_macros(self, out, test_for_coveragep = 0):
        #print normalized_trans
        seen = []
        out.write("\n\n##################### Allow Statements ##################### \n\n")
        for nt in normalized_trans.values():
            if nt.domain_context.find('polgen_temp') != -1 or nt.range_context.find('polgen_temp') != -1:
                continue
            if not test_for_coveragep or not nt.is_covered():
                if nt.action != 'spawn':
                    if nt.action in ['activate', 'exec']:
                        found = [nt.domain_context, nt.range_context]
                        if found not in seen: # avoiding duplicates
                            seen.append(found)
                            if nt.domain_context != nt.range_context:
                                out.write("can_exec(" + nt.domain_context.split(':')[-1] + ", " \
                                  + nt.range_context.split(':')[-1] + ") \n\n")
                    else:
                        out.write(str(nt) + '\n')
                    out.write('#         because \n')
                    for e in nt.evidence_for:
                        out.write('#            ' + str(e) + '\n')

the_no_macro_strategy = NoMacros()

def gen_pattern_instances2(out, recog, instance):
    """ Generate a set of macro statements - one for each path thru the instance tree"""
    seen = []
    violatorp = 0
    if recog.detection == 'violator':
            violatorp = 1
    pattern = recog.pattern
    spec_evidence_dict = {}
    for individual in walk.instance2individuals(instance):
            preal = presentations.create_patternR(recog, individual)
            spec = preal.spec()
            #pattern_assertion_factory(spec,preal, violatorp)
            if spec in spec_evidence_dict.keys():
                spec_evidence_dict[spec].append(individual)
            else:
                spec_evidence_dict[spec] = [individual]
    for spec in spec_evidence_dict.keys():
        out.write(spec)
        for e in spec_evidence_dict[spec]:
            out.write('#          because of  ' + str(e[:-1]) + '\n')


class WithMacros (GenerationStrategy):
    def do_allows_and_macros(self, strm, test_for_coveragep):
        the_no_macro_strategy.do_allows_and_macros(strm, 1)
        strm.write("##################### Pattern Instances #################### \n\n")
        for recog in SPECIFIERS:
            if len(PRGM.pattern_instances[recog.__name__]) != 0:
                strm.write('#        ' + recog.pattern.name + ' instances observed by ' + recog.__name__ + '\n')
                for instance in PRGM.pattern_instances[recog.__name__]:
                    gen_pattern_instances2(strm, recog, instance)
                    strm.write(' \n')

class WithMacrosForSurvivorsOnly (GenerationStrategy):
    def do_allows_and_macros(self):
        return "done"

gen_strategy = WithMacros()

def clean_up ():
    for r in resources_dict.values():
        pos = r.security_context.find(r.default_type)
        #print r.default_type
        if pos != -1: # default still in place
            r.security_context = r.security_context[0:pos] + PRGM.name + r.security_context[pos + len(r.default_type):]
    setup_normalized_transitions() # re-do with new types
            
def do_policy_generationCB():
    items = [f for f in resources_dict.values() if f.__class__.__name__ == 'File']
    data = [[f.name, f.sc_extent] for f in items if (len(f.type_contributors) != 0) \
            or f.sc_from == 'typegen']
    clean_up()
    do_fc_generation(data) 
    gen_strategy.do_te_generation()

