from patterns import *
from recog_domain import *

class RecogMediator (SpecObj):
    """ fails if any potential colleagues are in sending_to of another"""
    pattern = Mediator
    detection = "correct"
    inputs = ['colleagues = []']
    collections = [Recog('r2 is_an element from nodes_in \
                    [c.range for c in obj.out_edges]',
                         [Recog('r3 is_an element from nodes_in \
                          [c.domain for c in obj.in_edges]',
                                [Recog('colleagues is_a set from nodes_in \
                                 [x for x in [c.range for c in r2.out_edges] if x in [c.domain for c in r3.in_edges]]')])])]
    stoppers = ["Exists([pair for pair in cross_product(colleagues) if pair[1] in pair[0].sending_to])",
                "len(colleagues) < 2"]
    focus_name = "mediator"

class RecogSingleProcessController (SpecObj):
    pattern = SingleProcessMediator
    detection = "correct"
    collections = [Recog('r1 is_a set from nodes_in \
                    [t.range for t in obj.out_edges if \
                    t.range in [x.domain for x in obj.in_edges]]')]
    stoppers = ["len(r1) > 1"]
    focus_name = "mediator"

class RecogSingleProcessMediator (SpecObj):
    pattern = SingleProcessMediator
    detection = "correct"
    collections = [Recog('r1 is_a set from nodes_in \
                    [t.range for t in obj.out_edges if \
                    t.range in [x.domain for x in obj.in_edges] and \
                    t.range.__class__.__name__ == "File"]')]
    stoppers = ["len(r1) < 2"]
    focus_name = "mediator"

class RecogLogger1 (SpecObj):
    pattern = Logger
    detection = "correct"
    focus_name = "application"
    collections = [Recog('log_file is_a set from nodes_in \
    [t.range for t in obj.out_edges if t.range.name.find("var/log") != -1]')]

class RecogLogger2 (SpecObj):
    pattern = Logger
    detection = "almost correct"
    focus_name = "application"
    collections = [Recog('log_file is_a set from nodes_in \
    [t.range for t in obj.out_edges if len(t.range.in_edges) == 1 \
    and len(t.range.out_edges) == 0]')]

class RecogConfig1(SpecObj):
    pattern = Config
    detection = "correct"
    focus_name = "application"
    collections = [Recog('config_file is_a set from nodes_in \
    [t.domain for t in obj.in_edges if config_namep(t.domain.name)]')]

class RecogCanCheckSelinux (SpecObj):
    pattern = CanCheckSelinux
    detection = "correct"
    focus_name = "main"
    collections = [Recog('filesystems is_an element from nodes_in \
    [t.domain for t in obj.in_edges if t.domain.name.find("/proc/filesystems") != -1]',
                         [Recog('current is_a set from nodes_in \
    [t.domain for t in obj.in_edges \
    if t.domain.name.find("/proc/self/attr/current") != -1]')])]

class RecogInterpreter (SpecObj):
    pattern = Interpreter
    detection = "correct"
    focus_name = "application"
    collections = [Recog('code_file is_an element from nodes_in \
    [t.domain for t in obj.in_edges if t.action == "exec" and t.domain.name.find(".py") != -1]',
                         [Recog('interpreter is_a set from nodes_in \
    [t.domain for t in obj.in_edges \
    if t.domain != code_file and t.action == "exec"]')])]

class RecogPipe (SpecObj):
    pattern = Pipe
    detection = "correct"
    focus_name = "parent"
    collections = [Recog('pipe is_an element from nodes_in \
    [t.range for t in obj.out_edges if t.range.__class__.__name__ == "Fifo_file"]',
                         [Recog('child is_a set from nodes_in \
    [t.range for t in pipe.out_edges \
    if obj == pipe.creator]')])]

class RecogClientOnly (SpecObj):
    pattern = Client
    detection = "correct"
    focus_name = "client"
    collections = [Recog('socket is_an element from nodes_in \
    [t.range for t in obj.out_edges if t.range.__class__.__name__ == "Unix_Stream_Socket" and not Exists(t.range.out_edges)]')]

class RecogClientServer (SpecObj):
    pattern = ClientServer
    detection = "correct"
    focus_name = "client"
    collections = [Recog('socket is_an element from nodes_in \
    [t.range for t in obj.out_edges if t.range.__class__.__name__ == "Unix_Stream_Socket"]',
                         [Recog('server is_a set from nodes_in \
    [t.range for t in socket.out_edges if t.range != obj and \
     Exists([a for a in t.range.with_accepts if a.listening_on() == socket.name])]')])]

##class RecogClientServer (SpecObj):
## this only returns client processes for which there is a client link to some other external process
##    pattern = ClientServer
##    detection = "correct"
##    focus_name = "client"
##    search_set = "get_fillers('RecogClientOnly', 'client')"
##    collections = [Recog('socket is_an element from nodes_in \
##    [t.range for t in obj.out_edges if t.range.__class__.__name__ == "Unix_Stream_Socket"]',
##                         [Recog('server is_a set from nodes_in \
##    [t.range for t in socket.out_edges if t.range != obj and \
##     Exists([a for a in t.range.with_accepts if a.listening_on() == socket.name])]')])]
##

class RecogS2S (SpecObj):
    pattern = ClientServer
    detection = "correct"
    focus_name = "server"
    collections = [Recog('socket is_an element from nodes_in \
    [t.range for t in obj.out_edges if t.range.__class__.__name__ == "Unix_Stream_Socket"]',
                         [Recog('client is_a set from nodes_in \
    [t.range for t in socket.out_edges if  t.range == obj and \
    Exists([a for a in t.range.with_accepts if a.listening_on() == socket.name])]')])]

class RecogLibSelinuxInit (SpecObj):
    pattern = LibSelinuxInit
    detection = "correct"
    focus_name = "application"
    collections = [Recog('mounts is_an element from nodes_in \
    [t for t in obj.in_edges if obj_name_contains_one_of(t.domain, ["proc/mounts", "etc/selinux/config"])]',
                                [Recog('new_access is_a set from nodes_in \
                                [t.not_obj(obj) for t in (obj.in_edges + obj.out_edges) if t != mounts \
                                and obj_name_contains_one_of(t.not_obj(obj), ["etc/selinux/", "proc/mounts"])]')])]

class RecogSharedLibrary (SpecObj):
    pattern = SharedLibrary
    detection = "correct"
    focus_name = "application"
    collections = [Recog('lib_read is_a set from nodes_in \
    [t.domain for t in obj.in_edges if t.domain.name.find(".so") != -1]')]

class RecogTempUse (SpecObj):
    pattern = TempUse
    detection = "correct"
    focus_name = "application"
    collections = [Recog('temp_file is_a set from nodes_in \
    [t.range for t in obj.out_edges if t.range.name.find("tmp/") != -1]')]

class RecogExecutable (SpecObj):
    pattern = Executable
    detection = "correct"
    focus_name = "application"
    collections = [Recog('file is_an element from nodes_in \
    [t.range for t in obj.out_edges if t.action == "exec"]',
                         [Recog('process is_an element from nodes_in \
                         [y.range for y in flatten([x.out_edges for x in [t.range for t in obj.out_edges \
                         if t.action == "activate"]])]')])]
    
class RecogConfig2(SpecObj):
    pattern = Config
    detection = "almost correct"
    focus_name = "application"
    collections = [Recog('config_file is_a set from nodes_in \
    [t.domain for t in obj.in_edges if len(t.domain.out_edges) == 1 \
    and len(t.domain.in_edges) == 0]')]

##class RecogAlmostMediator (SpecObj):
##    pattern = Mediator
##    detection = "almost correct"
##    inputs = ['colleagues = []']
##    collections = [Recog('colleagues is_a set from nodes_in \
##                        [x for x in intersect(obj.receiving_from, obj.sending_to)]')]
##    stoppers = ["len(colleagues) < 2"]
##    focus_name = "mediator"

class RecogPipeline(SpecObj):
    pattern = Pipeline
    detection = "correct"
    collections = [Recog("r1 is_an element from nodes_in \
    [c.range for c in obj.out_edges if not config_namep(c.range.name)\
    and len(c.range.in_edges) == 1 and c.action == 'write']", \
                         [Recog("middle is_an element from nodes_in \
                         [c.range for c in r1.out_edges if c.range  != obj and c.action == 'read' and c.domain not in obj.receiving_from]", \
                                [Recog("r2 is_an element from nodes_in \
                                [c.range for c in middle.out_edges if not config_namep(c.range.name) \
                                and c.range != r1 and len(c.range.in_edges) == 1 and c.action == 'write']", \
                                       [Recog("end is_a set from nodes_in \
                                       [c.range for c in  r2.out_edges if c.range not in obj.receiving_from \
                                       and c.range != obj and c.range != middle and c.action == 'read' and c.range not in middle.receiving_from]")])])])]
    focus_name = "begin"

class RecogPipelineObsolete (SpecObj):
    """Allows a configuration file to be part of the pipeline"""
    pattern = Pipeline
    detection = "correct"
    collections = [Recog("r1 is_an element from nodes_in \
    [c.range for c in obj.out_edges if  len(c.range.in_edges) == 1 and c.action == 'write']", \
                         [Recog("middle is_an element from nodes_in \
                         [c.range for c in r1.out_edges if c.range  != obj and c.action == 'read' and c.domain not in obj.receiving_from]", \
                                [Recog("r2 is_an element from nodes_in \
                                [c.range for c in middle.out_edges if  c.range != r1 and len(c.range.in_edges) == 1 and c.action == 'write']", \
                                       [Recog("end is_a set from nodes_in \
                                       [c.range for c in  r2.out_edges if c.range not in obj.receiving_from \
                                       and c.range != obj and c.range != middle and c.action == 'read' and c.range not in middle.receiving_from]")])])])]
    focus_name = "begin"

class RecogFlawedIntegrityPipeline (SpecObj):
    pattern = Pipeline
    detection = "violator"
    collections = [Recog("r1 is_an element from nodes_in \
    [c.range for c in obj.out_edges if c.action == 'write']", \
                         [Recog("middle is_an element from nodes_in \
                         [c.range for c in r1.out_edges if c.range  != obj and c.action == 'read' and c.domain not in obj.receiving_from]", \
                                [Recog("r2 is_an element from nodes_in \
                                [c.range for c in middle.out_edges if c.range != r1 \
                                and (len(c.range.in_edges) > 1 or len(r1.in_edges) > 1)and c.action == 'write']", \
                                       [Recog("end is_a set from nodes_in \
                                       [c.range for c in  r2.out_edges if c.range not in obj.receiving_from \
                                       and c.range != obj and c.action == 'read' and c.range != middle and c.range not in middle.receiving_from]")])])])]
    focus_name = "begin"

class RecogProxy(SpecObj):
    pattern = Proxy
    detection  = "correct"
    collections = [Recog('r2 is_an element from nodes_in \
                    [c.range for c in obj.out_edges if c.action != "activate"]',
                         [Recog('service is_an element from nodes_in \
                                 [c.range for c in r2.out_edges if c.range != obj ]',
                                [Recog('r3 is_an element from nodes_in \
                          [c.domain for c in obj.in_edges if \
                                           c.domain in [x.range for x in service.out_edges]]',
                                       [Recog('client is_a set from nodes_in [x for x in obj.receiving_from \
                                          if x not in service.receiving_from and x != obj and x != service]')])])])]
    focus_name = "proxy"

class RecogHubAndSpoke(SpecObj):
    pattern = HubAndSpoke
    detection  = "correct"
    collections = [Recog('r2 is_an element from nodes_in \
                    [c.range for c in obj.out_edges if c.action == "activate"]',
                         [Recog('colleague is_an element from nodes_in \
                                 [c.range.executing[0] for c in r2.out_edges if len(c.range.executing) > 0]',
                                [Recog('r3 is_an element from nodes_in \
                          [c.range for c in obj.out_edges if c.action == "activate" and c.range != r2]',
                                       [Recog('center is_an element from nodes_in [c.range.executing[0] for c in r3.out_edges \
                                       if len(c.range.executing) > 0 and len(how_related(c.range, colleague.executed_by[0])) > 1]')])])])]
    focus_name = "initiator"

class RecogHubAndSpoke(SpecObj):
    pattern = HubAndSpoke
    detection  = "correct"
    collections = [Recog('r2 is_an element from nodes_in \
                    [c.range for c in obj.out_edges if c.action == "activate"]',
                         [Recog('colleague is_an element from nodes_in \
                                 [c.range.executing[0] for c in r2.out_edges if len(c.range.executing) > 0]', # and len(c.range.executing[0].with_accepts) == 0]',
                                [Recog('r3 is_an element from nodes_in \
                          [c.range for c in obj.out_edges if c.action == "activate" and c.range != r2]',
                                       [Recog('center is_an element from nodes_in [c.range.executing[0] for c in r3.out_edges \
                                       if len(c.range.executing) > 0 and len(c.range.executing[0].with_accepts) > 0 and \
                                              len(how_related(colleague, c.range.executing[0])) == 1]')])])])]
    focus_name = "initiator"

    
class RecogWebServerComponent (SpecObj):
    pattern = WebServerComponent
    detection = "correct"
    search_set = '[x for x in resources_dict.values() if x.is_process() and Exists(x.executing)]'
    collections = [Recog('r1 is_an element from nodes_in \
    [c.domain for c in obj.in_edges]',
                         [Recog('shell is_an element from nodes_in \
                         [c.domain for c in r1.in_edges if c.domain != obj]',
                                [Recog('r2 is_an element from nodes_in \
                                  [c.range for c in shell.out_edges]',
                                       [Recog('r3 is_an element from nodes_in \
                                          [c.domain for c in shell.in_edges]',
                                              [Recog ('script is_a set from nodes_in \
                                              [c for c in shell.executing if c in obj.executing and \
                                                      c not in [obj, shell] and c in intersect([x.domain for x in r3.in_edges], \
                                                                     [x.range for x in r2.out_edges])]')])])])])]
    focus_name = "server"

class RecogActiveObject (SpecObj):
    pattern = ActiveObject
    detection = "correct"
    collections = [Recog("order is_an element from nodes_in \
    [c.range for c in obj.out_edges if len(c.range.in_edges) == 1]", \
                         [Recog("service is_an element from nodes_in \
                         [c.range for c in order.out_edges if c.range  != obj and c.domain in obj.receiving_from]", \
                                [Recog("filled_order is_an element from nodes_in \
                                [c.range for c in service.out_edges if c.range != order and obj in [x.range for x in c.range.out_edges]]" \
                                       )])])]
    focus_name = "proxy"

# note: following list is order dependent
SPECIFIERS = [RecogHubAndSpoke, RecogExecutable, RecogPipeline, RecogClientOnly, RecogClientServer, RecogS2S, RecogPipe, RecogTempUse, RecogInterpreter, RecogCanCheckSelinux, RecogSharedLibrary, RecogLibSelinuxInit, RecogLogger1, \
              RecogConfig1, RecogActiveObject, RecogFlawedIntegrityPipeline,  \
              RecogSingleProcessMediator, RecogSingleProcessController, RecogProxy, RecogWebServerComponent, RecogMediator]
